--[[
    Calendar_Broker by LordFarlander
    Version 3.0.17
    $Revision: 17 $

    A conversion of Calendar_Fubar by wish
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Calendar_Broker" );

CalendarBroker = LibStub( "AceAddon-3.0" ):NewAddon( "Calendar_Broker", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0" );
local qtip = LibStub( "LibQTip-1.0" );

CalendarBroker.vars = {};

function CalendarBroker:Update()
    self:OnDataUpdate();
    self:OnTextUpdate();
    self:UpdateTooltip();
end--CalendarBroker:Update()

function CalendarBroker:UpdateTooltip()
    if( self.vars.qtip ) then
        self:OnTooltipUpdate();
    end--if
end--CalendarBroker:UpdateTooltip()

function CalendarBroker:OnDataUpdate()
    self:ProcessWorldEvents();
end--CalendarBroker:OnDataUpdate()

function CalendarBroker:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local defaults = {
        profile = {
            ShowDayOfWeek     = false,
            UseGEMdatabase    = true,
            DebugMessages     = false,
            DateFormat_US     = true,
            TimeFormat_12h    = true,
            MaxNumberOfDays   = 30,
            ServerTimeOffset  = 0,
            ZuluOffset        = 0,
            GUILDEventColor   = { a = 1.0, r = 0.0, g = 1.0, b = 0.0 },
            HOLIDAYEventColor = { a = 1.0, r = 0.0, g = 1.0, b = 1.0 },
            PLAYEREventColor  = { a = 1.0, r = 1.0, g = 1.0, b = 0.0 },
            RAIDEventColor    = { a = 1.0, r = 0.0, g = 0.0, b = 1.0 },
            SYSTEMEventColor  = { a = 1.0, r = 1.0, g = 0.0, b = 0.0 },
            ShowTimes         = false,
            ShowNumberDays    = true,
            ShowDurations     = false,
        },
    };
    local options = {
        type = "group",
        handler = self,
        args = {
            ShowDayOfWeek = {
                type = "toggle",
                order = 1,
                name = L["Show Day of the Week"],
                desc = L["Show Day of the Week"],
                width = "double",
                get = function( info )
                          return info.handler.db.profile.ShowDayOfWeek;
                      end,
                set = function( info, value )
                          info.handler.db.profile.ShowDayOfWeek = value;
                          info.handler:Update();
                      end,
            },
            UseGEMdatabase = {
                type = "toggle",
                order = 2,
                name = L["Use GEM database"],
                desc = L["Use GEM database"],
                width = "double",
                get = function( info )
                          return info.handler.db.profile.UseGEMdatabase;
                      end,
                set = function( info, value )
                          info.handler.db.profile.UseGEMdatabase = value;
                          info.handler:Update();
                      end,
            },
            DateFormat_US = {
                type = "toggle",
                order = 3,
                name = L["Date format USA"],
                desc = L["MM/DD/YYYY vs DD/MM/YYYY"],
                width = "double",
                get = function( info )
                          return self.db.profile.DateFormat_US;
                      end,
                set = function( info, value )
                          info.handler.db.profile.DateFormat_US = value;
                          info.handler:Update();
                      end,
            },
            TimeFormat_12h = {
                type = "toggle",
                order = 4,
                name = L["Time format 12 hour"],
                desc = L["12 hour time (AM/PM) vs 24 hour time"],
                width = "double",
                get = function( info )
                          return self.db.profile.TimeFormat_12h;
                      end,
                set = function( info, value )
                          info.handler.db.profile.TimeFormat_12h = value;
                          info.handler:Update();
                      end,
            },
            MaxNumberOfDays = {
                type = "range",
                order = 5,
                name = L["Number of days shown in Upcoming Events"],
                min = 7,
                max = 60,
                bigStep = 1,
                desc = L["Number of days shown in Upcoming Events"],
                width = "double",
                get = function( info )
                          return self.db.profile.MaxNumberOfDays;
                      end,
                set = function( info, value )
                          info.handler.db.profile.MaxNumberOfDays = value;
                          info.handler:Update();
                      end,
            },
            ShowNumberDays = {
                type = "toggle",
                order = 6,
                name = L["Show number of days until events"],
                desc = L["Show number of days until events"],
                width = "double",
                get = function( info )
                          return self.db.profile.ShowNumberDays;
                      end,
                set = function( info, value )
                          info.handler.db.profile.ShowNumberDays = value;
                          info.handler:Update();
                      end,
            },
            ShowTimes = {
                type = "toggle",
                order = 7,
                name = L["Show date and time event starts or ends"],
                desc = L["Show date and time event starts or ends"],
                width = "double",
                get = function( info )
                          return self.db.profile.ShowTimes;
                      end,
                set = function( info, value )
                          info.handler.db.profile.ShowTimes = value;
                          info.handler:Update();
                      end,
            },
            ShowDurations = {
                type = "toggle",
                order = 8,
                name = L["Show duration of events"],
                desc = L["Show duration of events"],
                width = "double",
                get = function( info )
                          return self.db.profile.ShowDurations;
                      end,
                set = function( info, value )
                          info.handler.db.profile.ShowDurations = value;
                          info.handler:Update();
                      end,
            },
            GUILDEventColor = {
                type = "color",
                order = 9,
                name = L["Guild event color"],
                desc = L["Guild event color"],
                hasAlpha = false,
                width = "double",
                get = function( info )
                          return info.handler.db.profile.GUILDEventColor.r, info.handler.db.profile.GUILDEventColor.g, info.handler.db.profile.GUILDEventColor.b, info.handler.db.profile.GUILDEventColor.a;
                      end,
                set = function( info, r, g, b, a )
                          info.handler.db.profile.GUILDEventColor.r = r;
                          info.handler.db.profile.GUILDEventColor.g = g;
                          info.handler.db.profile.GUILDEventColor.b = b;
                          info.handler.db.profile.GUILDEventColor.a = a;
                          info.handler:Update();
                      end,
            },
            HOLIDAYEventColor = {
                type = "color",
                order = 10,
                name = L["Holiday event color"],
                desc = L["Holiday event color"],
                hasAlpha = false,
                width = "double",
                get = function( info )
                          return info.handler.db.profile.HOLIDAYEventColor.r, info.handler.db.profile.HOLIDAYEventColor.g, info.handler.db.profile.HOLIDAYEventColor.b, info.handler.db.profile.HOLIDAYEventColor.a;
                      end,
                set = function( info, r, g, b, a )
                          info.handler.db.profile.HOLIDAYEventColor.r = r;
                          info.handler.db.profile.HOLIDAYEventColor.g = g;
                          info.handler.db.profile.HOLIDAYEventColor.b = b;
                          info.handler.db.profile.HOLIDAYEventColor.a = a;
                          info.handler:Update();
                      end,
            },
            PLAYEREventColor = {
                type = "color",
                order = 11,
                name = L["Player event color"],
                desc = L["Player event color"],
                hasAlpha = false,
                width = "double",
                get = function( info )
                          return info.handler.db.profile.PLAYEREventColor.r, info.handler.db.profile.PLAYEREventColor.g, info.handler.db.profile.PLAYEREventColor.b, info.handler.db.profile.PLAYEREventColor.a;
                      end,
                set = function( info, r, g, b, a )
                          info.handler.db.profile.PLAYEREventColor.r = r;
                          info.handler.db.profile.PLAYEREventColor.g = g;
                          info.handler.db.profile.PLAYEREventColor.b = b;
                          info.handler.db.profile.PLAYEREventColor.a = a;
                          info.handler:Update();
                      end,
            },
            RAIDEventColor = {
                type = "color",
                order = 12,
                name = L["Raid event color"],
                desc = L["Raid event color"],
                hasAlpha = false,
                width = "double",
                get = function( info )
                          return info.handler.db.profile.RAIDEventColor.r, info.handler.db.profile.RAIDEventColor.g, info.handler.db.profile.RAIDEventColor.b, info.handler.db.profile.RAIDEventColor.a;
                      end,
                set = function( info, r, g, b, a )
                          info.handler.db.profile.RAIDEventColor.r = r;
                          info.handler.db.profile.RAIDEventColor.g = g;
                          info.handler.db.profile.RAIDEventColor.b = b;
                          info.handler.db.profile.RAIDEventColor.a = a;
                          info.handler:Update();
                      end,
            },
            SYSTEMEventColor = {
                type = "color",
                order = 13,
                name = L["System event color"],
                desc = L["System event color"],
                hasAlpha = false,
                width = "double",
                get = function( info )
                          return info.handler.db.profile.SYSTEMEventColor.r, info.handler.db.profile.SYSTEMEventColor.g, info.handler.db.profile.SYSTEMEventColor.b, info.handler.db.profile.SYSTEMEventColor.a;
                      end,
                set = function( info, r, g, b, a )
                          info.handler.db.profile.SYSTEMEventColor.r = r;
                          info.handler.db.profile.SYSTEMEventColor.g = g;
                          info.handler.db.profile.SYSTEMEventColor.b = b;
                          info.handler.db.profile.SYSTEMEventColor.a = a;
                          info.handler:Update();
                      end,
            },
        },
    };

    self.db = LibStub( "AceDB-3.0" ):New( "CalendarBrokerDB", defaults );

    local profileOptions = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db );
--    self.db.RegisterCallback( self, "OnProfileChanged" );
--    self.db.RegisterCallback( self, "OnProfileReset" );

    AceConfigReg:RegisterOptionsTable( "CalendarBroker Main Options", options );
    AceConfigReg:RegisterOptionsTable( "CalendarBroker Profile", profileOptions );
    AceConfigDialog:AddToBlizOptions( "CalendarBroker Main Options", L["Calendar Broker"] );
    AceConfigDialog:AddToBlizOptions( "CalendarBroker Profile", L["Profile"], L["Calendar Broker"] );

    self.ldbobj = LibStub( "LibDataBroker-1.1" ):NewDataObject( L["Calendar Broker"], {
            type = "data source",
            icon = "Interface\\AddOns\\Calendar_Broker\\Artwork\\Calendar.blp",
            label = L["Calendar Broker"],
            text = L["Calendar Broker"], -- To get Button Bin to work correctly when autohiding text
            value = "0",
            suffix = L["events"],
            OnClick = function()
                          GameTimeFrame_OnClick();
                      end,
            OnEnter = function( frame )
                          if( frame and ( not InCombatLockdown() ) ) then
                              local tooltip;
                              local numcolumns = ( self.db.profile.ShowTimes and 1 or 0 ) + ( self.db.profile.ShowNumberDays and 1 or 0 ) + ( self.db.profile.ShowDurations and 1 or 0 );

                              if( numcolumns == 3 ) then
                                  tooltip = qtip:Acquire( "CalendarBroker_Tooltip", 4, "LEFT", "CENTER", "CENTER", "RIGHT" );
                              elseif( numcolumns == 2 ) then
                                  tooltip = qtip:Acquire( "CalendarBroker_Tooltip", 3, "LEFT", "CENTER", "RIGHT" );
                              elseif( numcolumns == 0 ) then
                                  tooltip = qtip:Acquire( "CalendarBroker_Tooltip", 1, "LEFT" );
                              else
                                  tooltip = qtip:Acquire( "CalendarBroker_Tooltip", 2, "LEFT", "RIGHT" );
                              end--if
                              self.vars.qtip = tooltip;
                              self:UpdateTooltip();
                              tooltip:SmartAnchorTo( frame );
                              tooltip:Show();
                          end--if
                      end,
            OnLeave = function( frame )
                          qtip:Release( self.vars.qtip );
                          self.vars.qtip = nil;
                      end,
        } );
end--CalendarBroker:OnInitialize()

function CalendarBroker:OnEnable()
    self:RegisterEvent( "CALENDAR_UPDATE_EVENT_LIST", "Update" );
    self.timerHandle = self:ScheduleRepeatingTimer( self.Update, 5 * 60, self );
    self:Update();
end--CalendarBroker:OnEnable()

function CalendarBroker:OnTextUpdate()
    local rText = "";
    local eventCount = self.events.EventsTodayCount;
    local lowestTime = 0;

    if( self.db.profile.ShowDayOfWeek ) then
        rText = date( "%A", time() );
    else
        rText = self:GetFormattedDate( time() );
    end--if
    if( eventCount > 0 ) then
        if( eventCount == 1 ) then
            rText = L["1 event"];
        else
            rText = L["%i events"]:format( eventCount );
        end--if
    end--if
    self.ldbobj.value = tostring( eventCount );

    for i, v in pairs( self.events.EventsToday ) do
        if( v.time2 < lowestTime ) then
            rText = v.title .. " " .. self.Time2Text( v.time1 );
            lowestTime = v.time2;
        end--if
        if( v.time1 < lowestTime ) then
            rText = v.title .. " " .. self.Time2Text( v.time1 );
            lowestTime = v.time1;
        end--if
    end--for
    self.ldbobj.text = rText;
end--CalendarBroker:OnTextUpdate()

function CalendarBroker:OnTooltipUpdate()
    local vars = self.vars;
    local tooltip = vars.qtip;
    local profile = self.db.profile;
    local ShowNumberDays = profile.ShowNumberDays;
    local ShowTimes = profile.ShowTimes;
    local ShowDurations = profile.ShowDurations;
    local numcolumns = ( ShowTimes and 1 or 0 ) + ( ShowNumberDays and 1 or 0 ) + ( ShowDurations and 1 or 0 );
    local nowTime = time();
    
    tooltip:Clear();
    if( numcolumns == 3 ) then
        tooltip:SetColumnLayout( 3, "LEFT", "CENTER", "CENTER", "RIGHT" );
    elseif( numcolumns == 2 ) then
        tooltip:SetColumnLayout( 3, "LEFT", "CENTER", "RIGHT" );
    elseif( numcolumns == 0 ) then
        tooltip:SetColumnLayout( 1, "LEFT" );
    else
        tooltip:SetColumnLayout( 2, "LEFT", "RIGHT" );
    end--if
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Calendar Broker"], tooltip:GetHeaderFont(), nil, tooltip:GetColumnCount() );
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );
    if( #self.events.EventsToday ) then
        line, column = tooltip:AddLine();
        line, column = tooltip:SetCell( line, column, L["Current Events:"] );
        if( ShowNumberDays ) then
            line, column = tooltip:SetCell( line, column, " " );
        end--if
        if( ShowTimes ) then
            line, column = tooltip:SetCell( line, column, L["Ends:"], nil, "CENTER" );
        end--if
        if( ShowDurations ) then
            line, column = tooltip:SetCell( line, column, L["Left:"], nil, "CENTER" );
        end--if
    
        for i, v in pairs( self.events.EventsToday ) do
            local color = self.db.profile[v.type .. "EventColor"] or { r = 1.0, g = 1.0, b = 1.0, a = 1.0, };

            if( v.time1 == 0 ) then
                line, column = tooltip:AddLine();
                line, column = tooltip:SetCell( line, column, ("|cff%02x%02x%02x%s|r"):format( color.r * 255, color.g * 255, color.b * 255, v.title ) );
                if( ShowNumberDays ) then
                    line, column = tooltip:SetCell( line, column, " " );
                end--if
                if( ShowTimes ) then
                    line, column = tooltip:SetCell( line, column, self:GetFormattedDate( v.time2 ) .. " " .. self:GetFormattedTime( v.time2 ) );
                end--if
                if( ShowDurations ) then
                    line, column = tooltip:SetCell( line, column, self.Time2Text( v.length - ( v.time2 - nowTime ) ) );
                end--if
            else
                line, column = tooltip:AddLine();
                line, column = tooltip:SetCell( line, column, ("|cff%02x%02x%02x%s|r"):format( color.r * 255, color.g * 255, color.b * 255, v.title ) );
                if( ShowNumberDays ) then
                    line, column = tooltip:SetCell( line, column, self.Time2Text( v.time1 ) );
                end--if
                if( ShowTimes ) then
                    line, column = tooltip:SetCell( line, column, self:GetFormattedDate( v.time2 ) .. " " .. self:GetFormattedTime( v.time2 ) );
                end--if
                if( ShowDurations ) then
                    line, column = tooltip:SetCell( line, column, self.Time2Text( v.length - ( v.time2 - nowTime ) ) );
                end--if
            end--if
        end--for
        line, column = tooltip:AddLine();
        line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );        
    end--if

    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Upcoming Events:"] );
    if( ShowNumberDays ) then
        line, column = tooltip:SetCell( line, column, L["Starts in:"] );
    end--if
    if( ShowTimes ) then
        line, column = tooltip:SetCell( line, column, L["Starts at:"], nil, "CENTER" );
    end--if
    if( ShowDurations ) then
        line, column = tooltip:SetCell( line, column, L["Length:"], nil, "CENTER" );
    end--if

    for i, v in pairs( self.events.EventsUpcoming ) do
        if( v.time1 <= self.db.profile.MaxNumberOfDays * CALENDARBROKER_SECPERDAY ) then
            local color = self.db.profile[v.type .. "EventColor"] or { r = 1.0, g = 1.0, b = 1.0, a = 1.0, };

            line, column = tooltip:AddLine();
            line, column = tooltip:SetCell( line, column, ("|cff%02x%02x%02x%s|r"):format( color.r * 255, color.g * 255, color.b * 255, v.title ) );
            if( ShowNumberDays ) then
                line, column = tooltip:SetCell( line, column, self.Time2Text( v.time1 ) );
            end--if
            if( ShowTimes ) then
                line, column = tooltip:SetCell( line, column, self:GetFormattedDate( v.time1 ) .. " " .. self:GetFormattedTime( v.time1 ) );
            end--if
            if( ShowDurations ) then
                line, column = tooltip:SetCell( line, column, self.Time2Text( v.length ) );
            end--if            
        end--if
    end--for

    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, date( "%A", nowTime ) .. "  " .. self:GetFormattedDate( nowTime ), nil, nil, tooltip:GetColumnCount() );
end--CalendarBroker:OnTooltipUpdate()


